package javax.imageio.metadata;

import org.w3c.dom.Node;
import java.security.AccessController;
import java.security.PrivilegedAction;

public abstract class IIOMetadata
{
    protected boolean standardFormatSupported;
    protected String nativeMetadataFormatName;
    protected String nativeMetadataFormatClassName;
    protected String[] extraMetadataFormatNames;
    protected String[] extraMetadataFormatClassNames;
    protected IIOMetadataController defaultController;
    protected IIOMetadataController controller;
    
    protected IIOMetadata() {
        this.nativeMetadataFormatName = null;
        this.nativeMetadataFormatClassName = null;
        this.extraMetadataFormatNames = null;
        this.extraMetadataFormatClassNames = null;
        this.defaultController = null;
        this.controller = null;
    }
    
    protected IIOMetadata(boolean standardFormatSupported, String nativeMetadataFormatName, String nativeMetadataFormatClassName, String[] array, String[] array2) {
        this.nativeMetadataFormatName = null;
        this.nativeMetadataFormatClassName = null;
        this.extraMetadataFormatNames = null;
        this.extraMetadataFormatClassNames = null;
        this.defaultController = null;
        this.controller = null;
        this.standardFormatSupported = standardFormatSupported;
        this.nativeMetadataFormatName = nativeMetadataFormatName;
        this.nativeMetadataFormatClassName = nativeMetadataFormatClassName;
        if (array != null) {
            if (array.length == 0) {
                throw new IllegalArgumentException("extraMetadataFormatNames.length == 0!");
            }
            if (array2 == null) {
                throw new IllegalArgumentException("extraMetadataFormatNames != null && extraMetadataFormatClassNames == null!");
            }
            if (array2.length != array.length) {
                throw new IllegalArgumentException("extraMetadataFormatClassNames.length != extraMetadataFormatNames.length!");
            }
            this.extraMetadataFormatNames = (String[])(String[])array.clone();
            this.extraMetadataFormatClassNames = (String[])(String[])array2.clone();
        }
        else if (array2 != null) {
            throw new IllegalArgumentException("extraMetadataFormatNames == null && extraMetadataFormatClassNames != null!");
        }
    }
    
    public boolean isStandardMetadataFormatSupported() {
        return this.standardFormatSupported;
    }
    
    public abstract boolean isReadOnly();
    
    public String getNativeMetadataFormatName() {
        return this.nativeMetadataFormatName;
    }
    
    public String[] getExtraMetadataFormatNames() {
        if (this.extraMetadataFormatNames == null) {
            return null;
        }
        return (String[])(String[])this.extraMetadataFormatNames.clone();
    }
    
    public String[] getMetadataFormatNames() {
        String nativeMetadataFormatName = this.getNativeMetadataFormatName();
        String s = this.isStandardMetadataFormatSupported() ? "javax_imageio_1.0" : null;
        String[] extraMetadataFormatNames = this.getExtraMetadataFormatNames();
        int n = 0;
        if (nativeMetadataFormatName != null) {
            ++n;
        }
        if (s != null) {
            ++n;
        }
        if (extraMetadataFormatNames != null) {
            n += extraMetadataFormatNames.length;
        }
        if (n == 0) {
            return null;
        }
        String[] array = new String[n];
        int n2 = 0;
        if (nativeMetadataFormatName != null) {
            array[n2++] = nativeMetadataFormatName;
        }
        if (s != null) {
            array[n2++] = s;
        }
        if (extraMetadataFormatNames != null) {
            for (int i = 0; i < extraMetadataFormatNames.length; ++i) {
                array[n2++] = extraMetadataFormatNames[i];
            }
            return array;
        }
        return array;
    }
    
    public IIOMetadataFormat getMetadataFormat(String s) {
        if (s == null) {
            throw new IllegalArgumentException("formatName == null!");
        }
        if (this.standardFormatSupported && s.equals("javax_imageio_1.0")) {
            return IIOMetadataFormatImpl.getStandardFormatInstance();
        }
        String nativeMetadataFormatClassName = null;
        if (s.equals(this.nativeMetadataFormatName)) {
            nativeMetadataFormatClassName = this.nativeMetadataFormatClassName;
        }
        else if (this.extraMetadataFormatNames != null) {
            int i = 0;
            while (i < this.extraMetadataFormatNames.length) {
                if (s.equals(this.extraMetadataFormatNames[i])) {
                    nativeMetadataFormatClassName = this.extraMetadataFormatClassNames[i];
                    break;
                }
                ++i;
            }
        }
        if (nativeMetadataFormatClassName == null) {
            throw new IllegalArgumentException("Unsupported format name");
        }
        try {
            ClassLoader loader = (ClassLoader)AccessController.doPrivileged(new PrivilegedAction() {
                
                @Override
                public Object run() {
                    return o.getClass().getClassLoader();
                }
            });
            Class<Object> clazz;
            try {
                clazz = Class.forName(nativeMetadataFormatClassName, true, loader);
            }
            catch (ClassNotFoundException ex) {
                ClassLoader loader2 = (ClassLoader)AccessController.doPrivileged(new PrivilegedAction() {
                    @Override
                    public Object run() {
                        return Thread.currentThread().getContextClassLoader();
                    }
                });
                try {
                    clazz = Class.forName(nativeMetadataFormatClassName, true, loader2);
                }
                catch (ClassNotFoundException ex2) {
                    clazz = Class.forName(nativeMetadataFormatClassName, true, ClassLoader.getSystemClassLoader());
                }
            }
            return (IIOMetadataFormat)clazz.getMethod("getInstance", new Class[0]).invoke(null, new Object[0]);
        }
        catch (Exception cause) {
            IllegalStateException o = new IllegalStateException("Can't obtain format");
            o.initCause(cause);
            throw o;
        }
    }
    
    public abstract Node getAsTree(String p0);
    
    public abstract void mergeTree(String p0, Node p1) throws IIOInvalidTreeException;
    
    protected IIOMetadataNode getStandardChromaNode() {
        return null;
    }
    
    protected IIOMetadataNode getStandardCompressionNode() {
        return null;
    }
    
    protected IIOMetadataNode getStandardDataNode() {
        return null;
    }
    
    protected IIOMetadataNode getStandardDimensionNode() {
        return null;
    }
    
    protected IIOMetadataNode getStandardDocumentNode() {
        return null;
    }
    
    protected IIOMetadataNode getStandardTextNode() {
        return null;
    }
    
    protected IIOMetadataNode getStandardTileNode() {
        return null;
    }
    
    protected IIOMetadataNode getStandardTransparencyNode() {
        return null;
    }
    
    private void append(IIOMetadataNode iioMetadataNode, IIOMetadataNode iioMetadataNode2) {
        if (iioMetadataNode2 != null) {
            iioMetadataNode.appendChild(iioMetadataNode2);
        }
    }
    
    protected final IIOMetadataNode getStandardTree() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("javax_imageio_1.0");
        this.append(iioMetadataNode, this.getStandardChromaNode());
        this.append(iioMetadataNode, this.getStandardCompressionNode());
        this.append(iioMetadataNode, this.getStandardDataNode());
        this.append(iioMetadataNode, this.getStandardDimensionNode());
        this.append(iioMetadataNode, this.getStandardDocumentNode());
        this.append(iioMetadataNode, this.getStandardTextNode());
        this.append(iioMetadataNode, this.getStandardTileNode());
        this.append(iioMetadataNode, this.getStandardTransparencyNode());
        return iioMetadataNode;
    }
    
    public void setFromTree(String s, Node node) throws IIOInvalidTreeException {
        this.reset();
        this.mergeTree(s, node);
    }
    
    public abstract void reset();
    
    public void setController(IIOMetadataController controller) {
        this.controller = controller;
    }
    
    public IIOMetadataController getController() {
        return this.controller;
    }
    
    public IIOMetadataController getDefaultController() {
        return this.defaultController;
    }
    
    public boolean hasController() {
        return this.getController() != null;
    }
    
    public boolean activateController() {
        if (!this.hasController()) {
            throw new IllegalStateException("hasController() == false!");
        }
        return this.getController().activate(this);
    }
}
